@@ -524,7 +524,7 @@ pathnamecmp (const char *a,
 
 /**
  * gtk_im_multicontext_append_menuitems:
- * @context: a #GtkIMMultiContext
+ * @context: a #GtkIMMulticontext
  * @menushell: a #GtkMenuShell
  * 
  * Add menuitems for various available input methods to a menu;
@@ -538,9 +538,11 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
   const GtkIMContextInfo **contexts;
   guint n_contexts, i;
   GSList *group = NULL;
-  GtkWidget *menuitem;
+  GtkWidget *menuitem, *system_menuitem;
+  const char *system_context_id; 
   
-  menuitem = gtk_radio_menu_item_new_with_label (group, Q_("input method menu|System"));
+  system_context_id = _gtk_im_module_get_default_context_id (context->priv->client_window);
+  system_menuitem = menuitem = gtk_radio_menu_item_new_with_label (group, C_("input method menu", "System"));
   if (!user_context_id)
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), TRUE);
   group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
@@ -550,6 +552,10 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
   gtk_widget_show (menuitem);
   gtk_menu_shell_append (menushell, menuitem);
 
+  menuitem = gtk_separator_menu_item_new ();
+  gtk_widget_show (menuitem);
+  gtk_menu_shell_append (menushell, menuitem);
+
   _gtk_im_module_list (&contexts, &n_contexts);
 
   for (i = 0; i < n_contexts; i++)
@@ -613,7 +619,18 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
       if ((user_context_id &&
            strcmp (contexts[i]->context_id, user_context_id) == 0))
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), TRUE);
-      
+
+      if (strcmp (contexts[i]->context_id, system_context_id) == 0)
+        {
+          GtkWidget *label;
+          char *text;
+
+          label = gtk_bin_get_child (GTK_BIN (system_menuitem));
+          text = g_strdup_printf (C_("input method menu", "System (%s)"), translated_name);
+          gtk_label_set_text (GTK_LABEL (label), text);
+          g_free (text);
+        }     
+ 
       group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
       
       g_object_set_data (G_OBJECT (menuitem), I_("gtk-context-id"),
@@ -628,5 +645,22 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
   g_free (contexts);
 }
 
+/**
+ * gtk_im_multicontext_get_context_id:
+ * @context: a #GtkIMMulticontext
+ *
+ * Gets the id of the currently active slave of the @context.
+ *
+ * Returns: the id of the currently active slave
+ *
+ * Since: 2.16
+ */
+const char *
+gtk_im_multicontext_get_context_id (GtkIMMulticontext *context)
+{
+  return context->context_id;
+}
+
+
 #define __GTK_IM_MULTICONTEXT_C__
 #include "gtkaliasdef.c"
