@@ -29,8 +29,6 @@
 #include <unistd.h>
 #endif
 
-#include <gdk/gdkscreen.h>
-
 #include "gtkstock.h"
 #include "gtkicontheme.h"
 #include "gtkiconfactory.h"
@@ -389,9 +387,9 @@ gtk_recent_chooser_default_constructor (GType                  type,
   		    G_CALLBACK (recent_view_popup_menu_cb), impl);
   g_signal_connect (impl->recent_view, "button-press-event",
   		    G_CALLBACK (recent_view_button_press_cb), impl);
-  g_signal_connect (impl->recent_view, "drag_begin",
+  g_signal_connect (impl->recent_view, "drag-begin",
 		    G_CALLBACK (recent_view_drag_begin_cb), impl);
-  g_signal_connect (impl->recent_view, "drag_data_get",
+  g_signal_connect (impl->recent_view, "drag-data-get",
 		    G_CALLBACK (recent_view_drag_data_get_cb), impl);
 
   g_object_set (impl->recent_view, "has-tooltip", TRUE, NULL);
@@ -927,9 +925,8 @@ static void
 gtk_recent_chooser_default_map (GtkWidget *widget)
 {
   GtkRecentChooserDefault *impl = GTK_RECENT_CHOOSER_DEFAULT (widget);
-  
-  if (GTK_WIDGET_CLASS (_gtk_recent_chooser_default_parent_class)->map)
-    GTK_WIDGET_CLASS (_gtk_recent_chooser_default_parent_class)->map (widget);
+
+  GTK_WIDGET_CLASS (_gtk_recent_chooser_default_parent_class)->map (widget);
 
   /* reloads everything */
   reload_recent_items (impl);
@@ -1612,7 +1609,7 @@ remove_selected_from_list (GtkRecentChooserDefault *impl)
     {
       gchar *msg;
    
-      msg = strdup (_("Could not remove item"));
+      msg = g_strdup (_("Could not remove item"));
       error_message (impl, msg, err->message);
       
       g_free (msg);
