@@ -20,7 +20,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
+#include <string.h>
+
+#undef GTK_DISABLE_DEPRECATED /* GtkTooltips */
+
 #include "gtktoolitem.h"
 #include "gtkmarshalers.h"
 #include "gtktoolshell.h"
@@ -30,11 +35,9 @@
 #include "gtkprivate.h"
 #include "gtkalias.h"
 
-#include <string.h>
-
 /**
  * SECTION:gtktoolitem
- * @short_description: The base class of widgets that can be added to #GtkToolShell
+ * @short_description: The base class of widgets that can be added to GtkToolShell
  *
  * #GtkToolItem<!-- -->s are widgets that can appear on a toolbar. To
  * create a toolbar item that contain something else than a button, use
@@ -199,7 +202,7 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
  * Return value: %TRUE if the signal was handled, %FALSE if not
  **/
   toolitem_signals[CREATE_MENU_PROXY] =
-    g_signal_new (I_("create_menu_proxy"),
+    g_signal_new (I_("create-menu-proxy"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkToolItemClass, create_menu_proxy),
@@ -224,7 +227,7 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
  * themselves accordingly.
  **/
   toolitem_signals[TOOLBAR_RECONFIGURED] =
-    g_signal_new (I_("toolbar_reconfigured"),
+    g_signal_new (I_("toolbar-reconfigured"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkToolItemClass, toolbar_reconfigured),
@@ -248,7 +251,7 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
  *   need to use this signal anymore.
  **/
   toolitem_signals[SET_TOOLTIP] =
-    g_signal_new (I_("set_tooltip"),
+    g_signal_new (I_("set-tooltip"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkToolItemClass, set_tooltip),
@@ -281,12 +284,11 @@ gtk_tool_item_finalize (GObject *object)
   GtkToolItem *item = GTK_TOOL_ITEM (object);
 
   g_free (item->priv->menu_item_id);
-  
+
   if (item->priv->menu_item)
     g_object_unref (item->priv->menu_item);
-  
-  if (G_OBJECT_CLASS (gtk_tool_item_parent_class)->finalize)
-    G_OBJECT_CLASS (gtk_tool_item_parent_class)->finalize (object);
+
+  G_OBJECT_CLASS (gtk_tool_item_parent_class)->finalize (object);
 }
 
 static void
@@ -525,7 +527,7 @@ gtk_tool_item_new (void)
 
 /**
  * gtk_tool_item_get_icon_size:
- * @tool_item: a #GtkToolItem:
+ * @tool_item: a #GtkToolItem
  * 
  * Returns the icon size used for @tool_item. Custom subclasses of
  * #GtkToolItem should call this function to find out what size icons
@@ -551,7 +553,7 @@ gtk_tool_item_get_icon_size (GtkToolItem *tool_item)
 
 /**
  * gtk_tool_item_get_orientation:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * 
  * Returns the orientation used for @tool_item. Custom subclasses of
  * #GtkToolItem should call this function to find out what size icons
@@ -578,7 +580,7 @@ gtk_tool_item_get_orientation (GtkToolItem *tool_item)
 
 /**
  * gtk_tool_item_get_toolbar_style:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * 
  * Returns the toolbar style used for @tool_item. Custom subclasses of
  * #GtkToolItem should call this function in the handler of the
@@ -621,7 +623,7 @@ gtk_tool_item_get_toolbar_style (GtkToolItem *tool_item)
 
 /**
  * gtk_tool_item_get_relief_style:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * 
  * Returns the relief style of @tool_item. See gtk_button_set_relief_style().
  * Custom subclasses of #GtkToolItem should call this function in the handler
@@ -649,7 +651,7 @@ gtk_tool_item_get_relief_style (GtkToolItem *tool_item)
 
 /**
  * gtk_tool_item_set_expand:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * @expand: Whether @tool_item is allocated extra space
  * 
  * Sets whether @tool_item is allocated extra space when there
@@ -677,7 +679,7 @@ gtk_tool_item_set_expand (GtkToolItem *tool_item,
 
 /**
  * gtk_tool_item_get_expand:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * 
  * Returns whether @tool_item is allocated extra space.
  * See gtk_tool_item_set_expand().
@@ -696,7 +698,7 @@ gtk_tool_item_get_expand (GtkToolItem *tool_item)
 
 /**
  * gtk_tool_item_set_homogeneous:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * @homogeneous: whether @tool_item is the same size as other homogeneous items
  * 
  * Sets whether @tool_item is to be allocated the same size as other
@@ -723,13 +725,13 @@ gtk_tool_item_set_homogeneous (GtkToolItem *tool_item,
 
 /**
  * gtk_tool_item_get_homogeneous:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * 
  * Returns whether @tool_item is the same size as other homogeneous
  * items. See gtk_tool_item_set_homogeneous().
  * 
  * Return value: %TRUE if the item is the same size as other homogeneous
- * item.s
+ * items.
  * 
  * Since: 2.4
  **/
@@ -808,7 +810,7 @@ gtk_tool_item_real_set_tooltip (GtkToolItem *tool_item,
 
 /**
  * gtk_tool_item_set_tooltip:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * @tooltips: The #GtkTooltips object to be used
  * @tip_text: text to be used as tooltip text for @tool_item
  * @tip_private: text to be used as private tooltip text
@@ -837,7 +839,7 @@ gtk_tool_item_set_tooltip (GtkToolItem *tool_item,
 
 /**
  * gtk_tool_item_set_tooltip_text:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * @text: text to be used as tooltip for @tool_item
  *
  * Sets the text to be displayed as tooltip on the item.
@@ -861,7 +863,7 @@ gtk_tool_item_set_tooltip_text (GtkToolItem *tool_item,
 
 /**
  * gtk_tool_item_set_tooltip_markup:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * @markup: markup text to be used as tooltip for @tool_item
  *
  * Sets the markup text to be displayed as tooltip on the item.
@@ -1041,7 +1043,7 @@ gtk_tool_item_get_visible_vertical (GtkToolItem *toolitem)
 
 /**
  * gtk_tool_item_retrieve_proxy_menu_item:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * 
  * Returns the #GtkMenuItem that was last set by
  * gtk_tool_item_set_proxy_menu_item(), ie. the #GtkMenuItem
@@ -1067,7 +1069,7 @@ gtk_tool_item_retrieve_proxy_menu_item (GtkToolItem *tool_item)
 
 /**
  * gtk_tool_item_get_proxy_menu_item:
- * @tool_item: a #GtkToolItem: 
+ * @tool_item: a #GtkToolItem 
  * @menu_item_id: a string used to identify the menu item
  * 
  * If @menu_item_id matches the string passed to
@@ -1126,7 +1128,7 @@ gtk_tool_item_rebuild_menu (GtkToolItem *tool_item)
 
 /**
  * gtk_tool_item_set_proxy_menu_item:
- * @tool_item: a #GtkToolItem:
+ * @tool_item: a #GtkToolItem
  * @menu_item_id: a string used to identify @menu_item
  * @menu_item: a #GtkMenuItem to be used in the overflow menu
  * 
